<head {{ with .Site.LanguageCode }}lang="{{ . }}"{{ end }}>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=0.9, maximum-scale=0.9" />
        {{- with .Site.Params.description -}}
    <meta name="description" content="{{ . }}">
    {{- end -}}
    {{ .Hugo.Generator }}

    <title>{{ if not .IsHome }}{{ .Title }} &mdash;{{ end }} {{ .Site.Title }}</title>

    <link rel="stylesheet" href="{{ "css/basic.css" | relURL }}" type="text/css" />
    <link rel="stylesheet" href="{{ "css/alabaster.css" | relURL }}" type="text/css" />
    {{ with .Site.Params.highlightjs -}}
        <link rel="stylesheet" href="{{ . | relURL }}" type="text/css" />
    {{- end }}

    {{ range .Site.Params.custom_css -}}
        <link rel="stylesheet" href="{{ . | relURL }}">
    {{- end }}

    {{ range .Site.Params.custom_css_absolute -}}
        <link rel="stylesheet" href="{{ . }}">
    {{- end }}

    {{ with .Site.Params.favicon -}}
        <link rel="shortcut icon" href="{{ . | relURL }}" type="image/x-icon"/>
    {{- else -}}
        <link rel="shortcut icon" href="{{ "favicon.ico" | relURL }}" type="image/x-icon"/>
    {{- end }}
</head>
