<div class="sphinxsidebar" role="navigation" aria-label="main navigation">
    <div class="sphinxsidebarwrapper">
        <p class="logo">
            <a href="/">
                {{ with .Site.Params.sidebar.logo -}}
                    <img class="logo" src="{{ . | relURL }}" alt="Logo"/>
                    {{- end }}
                    <h1 class="logo logo-name">{{ .Site.Params.name }}</h1>
            </a>
        </p>

        {{ with .Site.Params.description -}}
            <p class="blurb">{{ . | markdownify }}</p>
        {{- end }}

        {{ partial "sidebar/buttons" . }}
        {{ partial "sidebar/menu" . }}
    </div>
</div>
<div class="clearer"></div>
</div>
