<div class="footer">
    {{ if .Site.Params.footer.show_powered_by -}}
	Exported from <a href="https://raw.githubusercontent.com/jwiegley/use-package/master/use-package.org"><code>use-package.org</code></a>
        using
        {{ with .Params.creator }}
            <a href="https://www.gnu.org/software/emacs/">{{ . | replaceRE "\\(Org.*" "" }}</a> &plus;
            <a href="https://orgmode.org/">{{ . | replaceRE "Emacs.*\\(" "" | replaceRE "\\+ ox-.*" "" }}</a>
        {{ end }}
        and published using <a href="http://gohugo.io/">Hugo {{ .Hugo.Version }}</a>
	&plus; <a href="https://github.com/digitalcraftsman/hugo-alabaster-theme">Alabaster</a>
    {{ end }}
</div>

{{ if .Site.Params.github_banner -}}
<a href="https://github.com/{{ .Site.Params.github_user }}/{{ .Site.Params.github_repo }}" class="github">
    <img style="position: absolute; top: 0; right: 0; border: 0;" src="https://s3.amazonaws.com/github/ribbons/forkme_right_darkblue_121621.png" alt="Fork me on GitHub" class="github"/>
</a>
{{- end }}
